# -*- coding: utf-8 -*-
"""
***************************************************************************
    OTBChoiceWidget.py
    ------------------
    Date                 : June 2017
    Copyright            : (C) 2017 by CS Systemes d'Information (CS SI)
    Email                : rashad dot kanavath at c-s fr

***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

__author__ = 'Rashad Kanavath'
__date__ = 'June 2017'
__copyright__ = "(C) 2017,2018 by CS Systemes d'information (CS SI)"

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

from qgis.PyQt.QtWidgets import QComboBox
from qgis.core import (QgsProcessingParameterString,
                       QgsProcessingOutputString)
from processing.gui.wrappers import (WidgetWrapper,
                                    DIALOG_STANDARD,
                                    DIALOG_BATCH,
                                    DIALOG_MODELER)

class OTBChoiceWidgetWrapper(WidgetWrapper):
    def createWidget(self):
        widget = QComboBox()
        widget.addItems(self.param.options)
        if self.dialogType in(DIALOG_MODELER, DIALOG_STANDARD):
            widget.currentIndexChanged.connect(self.valueChanged)
        return widget

    def get_algorithm(self):
        if self.dialogType == DIALOG_MODELER:
            return self.dialog._alg
        else:
            return self.dialog.algorithm()

    def setWrapperVisible(self, name, visible):
        if self.dialogType == DIALOG_STANDARD:
            if self.dialog.mainWidget() is None:
               return
            self.dialog.mainWidget().wrappers[name].widget.setVisible(visible)
            if self.dialog.mainWidget().wrappers[name].label:
                self.dialog.mainWidget().wrappers[name].label.setVisible(visible)
        else:
            if name in self.dialog.wrappers:
                self.dialog.wrappers[name].widget.setVisible(visible)
                if self.dialog.wrappers[name].label:
                    self.dialog.wrappers[name].label.setVisible(visible)

    def valueChanged(self, value):
        for parameter in self.get_algorithm().parameterDefinitions():
            if not 'group_key' in parameter.metadata() or parameter.metadata()['group_key'] != self.param.name():
                 continue
            name = parameter.name()
            v = self.value() == parameter.metadata()['group_value']
            self.setWrapperVisible(name, v)

    def setValue(self, value):
        if value in self.param.options:
            index = self.param.options.index(value)
        else:
            index = int(value)
        self.widget.setCurrentIndex(index)

    def value(self):
        return self.widget.currentText()

    def postInitialize(self, wrappers):
        if self.dialogType == DIALOG_BATCH:
            return

        for parameter in self.get_algorithm().parameterDefinitions():
            if not 'group_key' in parameter.metadata() or parameter.metadata()['group_key'] != self.param.name():
                continue
            name = parameter.name()
            v = self.value() == parameter.metadata()['group_value']
            for wrapper in wrappers:
                if wrapper.param.name() == name:
                    wrapper.widget.setVisible(v)
                    if wrapper.label:
                        wrapper.label.setVisible(v)

from qgis.core import QgsProcessingParameterDefinition

class OTBParameterChoice(QgsProcessingParameterDefinition):

    def __init__(self, name='', description='', options=[], default=None, isSource=False,
                 multiple=False, optional=False):

        QgsProcessingParameterDefinition.__init__(self, name, description, default, optional)

        self.setMetadata({
            'widget_wrapper': {
                'class': 'otb.OTBChoiceWidget.OTBChoiceWidgetWrapper'}})
        self.options = options

        if default is not None:
            try:
                self.default = int(default)
            except:
                self.default = 0
            self.value = self.default

    def setValue(self, value):
        if value is None:
            self.value = 0
        else:
            self.value = int(value)
            return True
    
    def type(self):
        return 'OTBParameterChoice'

